@@ -19,15 +19,7 @@ package org.apache.maven.wagon;
  * under the License.
  */
 
-import java.io.File;
-import java.io.IOException;
-import java.security.NoSuchAlgorithmException;
-import java.text.SimpleDateFormat;
-import java.util.ArrayList;
-import java.util.Collections;
-import java.util.Iterator;
-import java.util.List;
-
+import org.apache.log4j.Logger;
 import org.apache.maven.wagon.authentication.AuthenticationException;
 import org.apache.maven.wagon.authentication.AuthenticationInfo;
 import org.apache.maven.wagon.authorization.AuthorizationException;
@@ -43,13 +35,24 @@ import org.codehaus.plexus.util.FileUtils;
 import org.easymock.AbstractMatcher;
 import org.easymock.MockControl;
 
+import java.io.File;
+import java.io.IOException;
+import java.security.NoSuchAlgorithmException;
+import java.text.SimpleDateFormat;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
+
 /**
  * @author <a href="mailto:jason@maven.org">Jason van Zyl</a>
- * @version $Id: WagonTestCase.java 745730 2009-02-19 05:15:51Z brett $
+ * @version $Id: WagonTestCase.java 1173679 2011-09-21 14:32:35Z olamy $
  */
 public abstract class WagonTestCase
     extends PlexusTestCase
 {
+    protected static Logger logger = Logger.getLogger( WagonTestCase.class );
+
+
     static final class ProgressArgumentMatcher
         extends AbstractMatcher
     {
@@ -127,11 +130,18 @@ public abstract class WagonTestCase
 
     /**
      * Protocol id of the Wagon to use, eg. <code>scp</code>, <code>ftp</code>
-     * 
+     *
      * @return the protocol id
      */
     protected abstract String getProtocol();
 
+    /**
+     * The number of the port which should get used to start the test server
+     *
+     * @return the port number for the test server
+     */
+    protected abstract int getTestRepositoryPort();
+
     // ----------------------------------------------------------------------
     // 1. Create a local file repository which mimic a users local file
     // Repository.
@@ -220,7 +230,7 @@ public abstract class WagonTestCase
 
     protected void message( String message )
     {
-        System.out.println( message );
+        logger.info( message );
     }
 
     // ----------------------------------------------------------------------
@@ -230,13 +240,16 @@ public abstract class WagonTestCase
     public void testWagon()
         throws Exception
     {
-        setupRepositories();
+        if ( supportsGetIfNewer() )
+        {
+            setupRepositories();
 
-        setupWagonTestingFixtures();
+            setupWagonTestingFixtures();
 
-        fileRoundTripTesting();
+            fileRoundTripTesting();
 
-        tearDownWagonTestingFixtures();
+            tearDownWagonTestingFixtures();
+        }
     }
 
     public void testWagonGetIfNewerIsNewer()
@@ -282,8 +295,7 @@ public abstract class WagonTestCase
     }
 
     private void getIfNewer( long timestamp, boolean expectedResult, int expectedSize )
-        throws Exception, NoSuchAlgorithmException, IOException, ConnectionException, AuthenticationException,
-        TransferFailedException, ResourceDoesNotExistException, AuthorizationException
+        throws Exception
     {
         Wagon wagon = getWagon();
 
@@ -305,13 +317,14 @@ public abstract class WagonTestCase
                                          int expectedSize )
         throws IOException
     {
-        if ( expectedResult == true )
+        if ( expectedResult )
         {
             verifyMock( progressArgumentMatcher, expectedSize );
 
             assertNotNull( "check checksum is not null", checksumObserver.getActualChecksum() );
 
-            assertEquals( "compare checksums", "6b144b7285ffd6b0bc8300da162120b9", checksumObserver.getActualChecksum() );
+            assertEquals( "compare checksums", "6b144b7285ffd6b0bc8300da162120b9",
+                          checksumObserver.getActualChecksum() );
 
             // Now compare the contents of the artifact that was placed in
             // the repository with the contents of the artifact that was
@@ -344,7 +357,7 @@ public abstract class WagonTestCase
         destFile.deleteOnExit();
 
         ProgressArgumentMatcher progressArgumentMatcher = null;
-        if ( expectedResult == true )
+        if ( expectedResult )
         {
             progressArgumentMatcher = replaceMockForGet( wagon, expectedSize );
         }
@@ -358,8 +371,9 @@ public abstract class WagonTestCase
     private void replaceMockForSkippedGetIfNewer( Wagon wagon, int expectedSize )
     {
         Resource resource = new Resource( this.resource );
-        mockTransferListener.transferInitiated( createTransferEvent( wagon, resource, TransferEvent.TRANSFER_INITIATED,
-                                                                     TransferEvent.REQUEST_GET, destFile ) );
+        mockTransferListener.transferInitiated(
+            createTransferEvent( wagon, resource, TransferEvent.TRANSFER_INITIATED, TransferEvent.REQUEST_GET,
+                                 destFile ) );
         resource = new Resource( this.resource );
         resource.setContentLength( getExpectedContentLengthOnGet( expectedSize ) );
         resource.setLastModified( getExpectedLastModifiedOnGet( testRepository, resource ) );
@@ -418,7 +432,7 @@ public abstract class WagonTestCase
     /**
      * Test for putting a directory with a destination that multiple directories deep, all of which haven't been
      * created.
-     * 
+     *
      * @throws Exception
      * @since 1.0-beta-2
      */
@@ -465,7 +479,7 @@ public abstract class WagonTestCase
 
     /**
      * Test that when putting a directory that already exists new files get also copied
-     * 
+     *
      * @throws Exception
      * @since 1.0-beta-1
      */
@@ -520,7 +534,7 @@ public abstract class WagonTestCase
 
     /**
      * Test that when putting a directory that already exists new files get also copied and destination is "."
-     * 
+     *
      * @throws Exception
      * @since 1.0-beta-1
      */
@@ -554,7 +568,7 @@ public abstract class WagonTestCase
 
             wagon.putDirectory( sourceFile, "." );
 
-            List resourceNames = new ArrayList( resources.length + 1 );
+            List<String> resourceNames = new ArrayList<String>( resources.length + 1 );
 
             resourceNames.add( resourceToCreate );
             for ( int i = 0; i < resources.length; i++ )
@@ -572,10 +586,10 @@ public abstract class WagonTestCase
 
     /**
      * Create a directory with a resource and check that the other ones don't exist
-     * 
+     *
      * @param wagon
      * @param resourceToCreate name of the resource to be created
-     * @param dirName directory name to create
+     * @param dirName          directory name to create
      * @throws Exception
      */
     protected void createDirectory( Wagon wagon, String resourceToCreate, String dirName )
@@ -584,13 +598,13 @@ public abstract class WagonTestCase
         writeTestFile( resourceToCreate );
     }
 
-    protected void assertResourcesAreInRemoteSide( Wagon wagon, List resourceNames )
+    protected void assertResourcesAreInRemoteSide( Wagon wagon, List<String> resourceNames )
         throws IOException, TransferFailedException, ResourceDoesNotExistException, AuthorizationException
     {
-        Iterator iter = resourceNames.iterator();
+        Iterator<String> iter = resourceNames.iterator();
         while ( iter.hasNext() )
         {
-            String resourceName = (String) iter.next();
+            String resourceName = iter.next();
 
             File destFile = FileTestUtils.createUniqueFile( getName(), resourceName );
 
@@ -602,10 +616,10 @@ public abstract class WagonTestCase
 
     /**
      * Assert that a resource does not exist in the remote wagon system
-     * 
-     * @param wagon wagon to get the resource from
+     *
+     * @param wagon        wagon to get the resource from
      * @param resourceName name of the resource
-     * @throws IOException if a temp file can't be created
+     * @throws IOException             if a temp file can't be created
      * @throws AuthorizationException
      * @throws TransferFailedException
      * @since 1.0-beta-1
@@ -712,7 +726,7 @@ public abstract class WagonTestCase
 
     /**
      * Test {@link Wagon#getFileList(String)}.
-     * 
+     *
      * @throws Exception
      * @since 1.0-beta-2
      */
@@ -726,7 +740,7 @@ public abstract class WagonTestCase
         String dirName = "file-list";
 
         String filenames[] =
-            new String[] { "test-resource.txt", "test-resource.pom", "test-resource b.txt", "more-resources.dat",
+            new String[]{ "test-resource.txt", "test-resource.pom", "test-resource b.txt", "more-resources.dat",
                 ".index.txt" };
 
         for ( int i = 0; i < filenames.length; i++ )
@@ -738,7 +752,7 @@ public abstract class WagonTestCase
 
         wagon.connect( testRepository, getAuthInfo() );
 
-        List list = wagon.getFileList( dirName );
+        List<String> list = wagon.getFileList( dirName );
         assertNotNull( "file list should not be null.", list );
         assertTrue( "file list should contain more items (actually contains '" + list + "').",
                     list.size() >= filenames.length );
@@ -747,7 +761,7 @@ public abstract class WagonTestCase
         {
             assertTrue( "Filename '" + filenames[i] + "' should be in list.", list.contains( filenames[i] ) );
         }
-        
+
         // WAGON-250
         list = wagon.getFileList( "" );
         assertNotNull( "file list should not be null.", list );
@@ -766,7 +780,7 @@ public abstract class WagonTestCase
 
     /**
      * Test {@link Wagon#getFileList(String)} when the directory does not exist.
-     * 
+     *
      * @throws Exception
      * @since 1.0-beta-2
      */
@@ -802,7 +816,7 @@ public abstract class WagonTestCase
 
     /**
      * Test for an existing resource.
-     * 
+     *
      * @throws Exception
      * @since 1.0-beta-2
      */
@@ -828,7 +842,7 @@ public abstract class WagonTestCase
 
     /**
      * Test for an invalid resource.
-     * 
+     *
      * @throws Exception
      * @since 1.0-beta-2
      */
@@ -882,16 +896,18 @@ public abstract class WagonTestCase
     protected ProgressArgumentMatcher replayMockForPut( String resourceName, String content, Wagon wagon )
     {
         Resource resource = new Resource( resourceName );
-        mockTransferListener.transferInitiated( createTransferEvent( wagon, resource, TransferEvent.TRANSFER_INITIATED,
-                                                                     TransferEvent.REQUEST_PUT, sourceFile ) );
+        mockTransferListener.transferInitiated(
+            createTransferEvent( wagon, resource, TransferEvent.TRANSFER_INITIATED, TransferEvent.REQUEST_PUT,
+                                 sourceFile ) );
         resource = new Resource( resourceName );
         resource.setContentLength( content.length() );
         resource.setLastModified( sourceFile.lastModified() );
-        mockTransferListener.transferStarted( createTransferEvent( wagon, resource, TransferEvent.TRANSFER_STARTED,
-                                                                   TransferEvent.REQUEST_PUT, sourceFile ) );
-        mockTransferListener.transferProgress( createTransferEvent( wagon, resource, TransferEvent.TRANSFER_PROGRESS,
-                                                                    TransferEvent.REQUEST_PUT, sourceFile ),
-                                               new byte[] {}, 0 );
+        mockTransferListener.transferStarted(
+            createTransferEvent( wagon, resource, TransferEvent.TRANSFER_STARTED, TransferEvent.REQUEST_PUT,
+                                 sourceFile ) );
+        mockTransferListener.transferProgress(
+            createTransferEvent( wagon, resource, TransferEvent.TRANSFER_PROGRESS, TransferEvent.REQUEST_PUT,
+                                 sourceFile ), new byte[]{ }, 0 );
         ProgressArgumentMatcher progressArgumentMatcher = new ProgressArgumentMatcher();
         mockTransferListenerControl.setMatcher( progressArgumentMatcher );
 
@@ -899,8 +915,9 @@ public abstract class WagonTestCase
         mockTransferListenerControl.setMatcher( MockControl.ALWAYS_MATCHER );
         mockTransferListenerControl.setVoidCallable( MockControl.ZERO_OR_MORE );
 
-        mockTransferListener.transferCompleted( createTransferEvent( wagon, resource, TransferEvent.TRANSFER_COMPLETED,
-                                                                     TransferEvent.REQUEST_PUT, sourceFile ) );
+        mockTransferListener.transferCompleted(
+            createTransferEvent( wagon, resource, TransferEvent.TRANSFER_COMPLETED, TransferEvent.REQUEST_PUT,
+                                 sourceFile ) );
 
         mockTransferListenerControl.replay();
         return progressArgumentMatcher;
@@ -975,15 +992,18 @@ public abstract class WagonTestCase
     protected ProgressArgumentMatcher replaceMockForGet( Wagon wagon, int expectedSize )
     {
         Resource resource = new Resource( this.resource );
-        mockTransferListener.transferInitiated( createTransferEvent( wagon, resource, TransferEvent.TRANSFER_INITIATED,
-                                                                     TransferEvent.REQUEST_GET, destFile ) );
+        mockTransferListener.transferInitiated(
+            createTransferEvent( wagon, resource, TransferEvent.TRANSFER_INITIATED, TransferEvent.REQUEST_GET,
+                                 destFile ) );
         resource = new Resource( this.resource );
         resource.setContentLength( getExpectedContentLengthOnGet( expectedSize ) );
         resource.setLastModified( getExpectedLastModifiedOnGet( testRepository, resource ) );
-        mockTransferListener.transferStarted( createTransferEvent( wagon, resource, TransferEvent.TRANSFER_STARTED,
-                                                                   TransferEvent.REQUEST_GET, destFile ) );
-        mockTransferListener.transferProgress( new TransferEvent( wagon, resource, TransferEvent.TRANSFER_PROGRESS,
-                                                                  TransferEvent.REQUEST_GET ), new byte[] {}, 0 );
+        TransferEvent te =
+            createTransferEvent( wagon, resource, TransferEvent.TRANSFER_STARTED, TransferEvent.REQUEST_GET, null );
+        mockTransferListener.transferStarted( te );
+        mockTransferListener.transferProgress(
+            new TransferEvent( wagon, resource, TransferEvent.TRANSFER_PROGRESS, TransferEvent.REQUEST_GET ),
+            new byte[]{ }, 0 );
         ProgressArgumentMatcher progressArgumentMatcher = new ProgressArgumentMatcher();
         mockTransferListenerControl.setMatcher( progressArgumentMatcher );
 
@@ -991,8 +1011,9 @@ public abstract class WagonTestCase
         mockTransferListenerControl.setMatcher( MockControl.ALWAYS_MATCHER );
         mockTransferListenerControl.setVoidCallable( MockControl.ZERO_OR_MORE );
 
-        mockTransferListener.transferCompleted( createTransferEvent( wagon, resource, TransferEvent.TRANSFER_COMPLETED,
-                                                                     TransferEvent.REQUEST_GET, destFile ) );
+        mockTransferListener.transferCompleted(
+            createTransferEvent( wagon, resource, TransferEvent.TRANSFER_COMPLETED, TransferEvent.REQUEST_GET,
+                                 destFile ) );
 
         mockTransferListenerControl.replay();
         return progressArgumentMatcher;
