@@ -19,30 +19,12 @@ package org.apache.maven.wagon.http;
  * under the License.
  */
 
-import java.io.File;
-import java.io.FileOutputStream;
-import java.io.IOException;
-import java.io.OutputStream;
-import java.net.URLDecoder;
-import java.util.Enumeration;
-import java.util.HashMap;
-import java.util.Map;
-import java.util.Properties;
-import java.util.zip.GZIPOutputStream;
-
-import javax.servlet.ServletException;
-import javax.servlet.ServletInputStream;
-import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpServletResponse;
-
-import org.apache.maven.wagon.ConnectionException;
 import org.apache.maven.wagon.FileTestUtils;
 import org.apache.maven.wagon.ResourceDoesNotExistException;
 import org.apache.maven.wagon.StreamingWagon;
 import org.apache.maven.wagon.StreamingWagonTestCase;
 import org.apache.maven.wagon.TransferFailedException;
 import org.apache.maven.wagon.Wagon;
-import org.apache.maven.wagon.authentication.AuthenticationException;
 import org.apache.maven.wagon.authentication.AuthenticationInfo;
 import org.apache.maven.wagon.authorization.AuthorizationException;
 import org.apache.maven.wagon.proxy.ProxyInfo;
@@ -51,9 +33,11 @@ import org.apache.maven.wagon.resource.Resource;
 import org.codehaus.plexus.util.FileUtils;
 import org.codehaus.plexus.util.IOUtil;
 import org.codehaus.plexus.util.StringOutputStream;
+import org.codehaus.plexus.util.StringUtils;
 import org.mortbay.jetty.Handler;
 import org.mortbay.jetty.HttpConnection;
 import org.mortbay.jetty.Request;
+import org.mortbay.jetty.Response;
 import org.mortbay.jetty.Server;
 import org.mortbay.jetty.handler.AbstractHandler;
 import org.mortbay.jetty.handler.HandlerCollection;
@@ -65,6 +49,26 @@ import org.mortbay.jetty.servlet.Context;
 import org.mortbay.jetty.servlet.DefaultServlet;
 import org.mortbay.jetty.servlet.ServletHolder;
 
+import javax.servlet.ServletException;
+import javax.servlet.ServletInputStream;
+import javax.servlet.http.HttpServletRequest;
+import javax.servlet.http.HttpServletResponse;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.OutputStream;
+import java.lang.reflect.Method;
+import java.net.URLDecoder;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.Properties;
+import java.util.zip.GZIPOutputStream;
+
 /**
  * @version $Id: LightweightHttpWagonTest.java 680764 2008-07-29 16:45:51Z brett $
  */
@@ -88,7 +92,7 @@ public abstract class HttpWagonTestCase
         FileUtils.deleteDirectory( repositoryDirectory );
         repositoryDirectory.mkdirs();
 
-        server = new Server( 10007 );
+        server = new Server( 0 );
 
         PutHandler putHandler = new PutHandler( repositoryDirectory );
         server.addHandler( putHandler );
@@ -98,6 +102,18 @@ public abstract class HttpWagonTestCase
         addConnectors( server );
 
         server.start();
+
+        testRepository.setUrl( getTestRepositoryUrl() );
+    }
+
+    @Override
+    protected final int getTestRepositoryPort()
+    {
+        if ( server == null )
+        {
+            return 0;
+        }
+        return server.getConnectors()[0].getLocalPort();
     }
 
     protected void createContext( Server server, File repositoryDirectory )
@@ -143,7 +159,40 @@ public abstract class HttpWagonTestCase
         addConnectors( server );
         server.start();
 
-        wagon.connect( new Repository( "id", getProtocol() + "://localhost:" + server.getConnectors()[0].getLocalPort() ) );
+        wagon.connect(
+            new Repository( "id", getProtocol() + "://localhost:" + server.getConnectors()[0].getLocalPort() ) );
+
+        wagon.getToStream( "resource", new StringOutputStream() );
+
+        wagon.disconnect();
+
+        server.stop();
+
+        assertEquals( "Maven-Wagon/1.0", handler.headers.get( "User-Agent" ) );
+    }
+
+    /**
+     * test set of User-Agent as it's done by aether wagon connector with using setHttpHeaders
+     */
+    public void testHttpHeadersWithCommonMethods()
+        throws Exception
+    {
+        Properties properties = new Properties();
+        properties.setProperty( "User-Agent", "Maven-Wagon/1.0" );
+
+        StreamingWagon wagon = (StreamingWagon) getWagon();
+
+        Method setHttpHeaders = wagon.getClass().getMethod( "setHttpHeaders", Properties.class );
+        setHttpHeaders.invoke( wagon, properties );
+
+        Server server = new Server( 0 );
+        TestHeaderHandler handler = new TestHeaderHandler();
+        server.setHandler( handler );
+        addConnectors( server );
+        server.start();
+
+        wagon.connect(
+            new Repository( "id", getProtocol() + "://localhost:" + server.getConnectors()[0].getLocalPort() ) );
 
         wagon.getToStream( "resource", new StringOutputStream() );
 
@@ -209,8 +258,7 @@ public abstract class HttpWagonTestCase
     }
 
     private void runTestGet( int status )
-        throws Exception, ConnectionException, AuthenticationException, TransferFailedException,
-        ResourceDoesNotExistException, AuthorizationException
+        throws Exception
     {
         StreamingWagon wagon = (StreamingWagon) getWagon();
 
@@ -278,8 +326,7 @@ public abstract class HttpWagonTestCase
     }
 
     private boolean runTestResourceExists( int status )
-        throws Exception, ConnectionException, AuthenticationException, TransferFailedException,
-        ResourceDoesNotExistException, AuthorizationException
+        throws Exception
     {
         StreamingWagon wagon = (StreamingWagon) getWagon();
 
@@ -318,7 +365,7 @@ public abstract class HttpWagonTestCase
     public void testGzipGet()
         throws Exception
     {
-        Server server = new Server( 10008 );
+        Server server = new Server( getTestRepositoryPort() );
 
         String localRepositoryPath = FileTestUtils.getTestOutputDir().toString();
         Context root = new Context( server, "/", Context.SESSIONS );
@@ -377,28 +424,55 @@ public abstract class HttpWagonTestCase
         ProxyInfo proxyInfo = createProxyInfo();
         proxyInfo.setUserName( "user" );
         proxyInfo.setPassword( "secret" );
-        TestHeaderHandler handler = new AuthorizingProxyHandler();
+        AuthorizingProxyHandler handler = new AuthorizingProxyHandler();
 
         runTestProxiedRequest( proxyInfo, handler );
 
         assertTrue( handler.headers.containsKey( "Proxy-Authorization" ) );
+
+        if ( supportProxyPreemptiveAuthentication() )
+        {
+            assertEquals( 200, handler.securityHandlerRequestReponses.get( 0 ).responseCode );
+        }
+        else
+        {
+            assertEquals( 407, handler.securityHandlerRequestReponses.get( 0 ).responseCode );
+            assertEquals( 200, handler.securityHandlerRequestReponses.get( 1 ).responseCode );
+        }
+
     }
 
     private void runTestProxiedRequest( ProxyInfo proxyInfo, TestHeaderHandler handler )
-        throws Exception, IOException, ConnectionException, AuthenticationException, ResourceDoesNotExistException,
-        TransferFailedException, AuthorizationException
+        throws Exception
     {
-        Server proxyServer = new Server( 10007 );
+        // what an UGLY hack!
+        // but apparently jetty needs some time to free up resources
+        // <5s: broken test :(
+        Thread.sleep( 5001L );
+
+        Server proxyServer = new Server( 0 );
 
         proxyServer.setHandler( handler );
+
         proxyServer.start();
 
-        proxyInfo.setPort( 10007 );
+        proxyInfo.setPort( proxyServer.getConnectors()[0].getLocalPort() );
+
+        System.out.println(
+            "start proxy on host/port " + proxyInfo.getHost() + "/" + proxyInfo.getPort() + " with non proxyHosts "
+                + proxyInfo.getNonProxyHosts() );
+
+        while ( !proxyServer.isRunning() || !proxyServer.isStarted() )
+        {
+            Thread.sleep( 10 );
+        }
 
         try
         {
             StreamingWagon wagon = (StreamingWagon) getWagon();
 
+            System.out.println( " wagon hashCode " + wagon.hashCode() );
+
             Repository testRepository = new Repository( "id", "http://www.example.com/" );
 
             String localRepositoryPath = FileTestUtils.getTestOutputDir().toString();
@@ -416,6 +490,8 @@ public abstract class HttpWagonTestCase
             }
             finally
             {
+                System.setProperty( "http.proxyHost", "" );
+                System.setProperty( "http.proxyPort", "" );
                 wagon.disconnect();
             }
         }
@@ -602,25 +678,6 @@ public abstract class HttpWagonTestCase
         return server;
     }
 
-    protected SecurityHandler createSecurityHandler()
-    {
-        Constraint constraint = new Constraint();
-        constraint.setName( Constraint.__BASIC_AUTH );
-        constraint.setRoles( new String[] { "admin" } );
-        constraint.setAuthenticate( true );
-
-        ConstraintMapping cm = new ConstraintMapping();
-        cm.setConstraint( constraint );
-        cm.setPathSpec( "/*" );
-
-        SecurityHandler sh = new SecurityHandler();
-        HashUserRealm hashUserRealm = new HashUserRealm( "MyRealm" );
-        hashUserRealm.put( "user", "secret" );
-        hashUserRealm.addUserToRole( "user", "admin" );
-        sh.setUserRealm( hashUserRealm );
-        sh.setConstraintMappings( new ConstraintMapping[] { cm } );
-        return sh;
-    }
 
     private String writeTestFileGzip( File parent, String child )
         throws IOException
@@ -629,18 +686,32 @@ public abstract class HttpWagonTestCase
         file.getParentFile().mkdirs();
         file.deleteOnExit();
         OutputStream out = new FileOutputStream( file );
-        out.write( child.getBytes() );
-        out.close();
+        try
+        {
+            out.write( child.getBytes() );
+        }
+        finally
+        {
+            out.close();
+        }
 
         file = new File( parent, child + ".gz" );
         file.deleteOnExit();
+        String content;
         out = new FileOutputStream( file );
         out = new GZIPOutputStream( out );
-        // write out different data than non-gz file, so we can
-        // assert the gz version was returned
-        String content = file.getAbsolutePath();
-        out.write( content.getBytes() );
-        out.close();
+        try
+        {
+            // write out different data than non-gz file, so we can
+            // assert the gz version was returned
+            content = file.getAbsolutePath();
+            out.write( content.getBytes() );
+        }
+        finally
+        {
+            out.close();
+        }
+
         return content;
     }
 
@@ -687,8 +758,7 @@ public abstract class HttpWagonTestCase
     }
 
     private void runTestPut( int status )
-        throws Exception, ConnectionException, AuthenticationException, TransferFailedException,
-        ResourceDoesNotExistException, AuthorizationException
+        throws Exception
     {
         StreamingWagon wagon = (StreamingWagon) getWagon();
 
@@ -762,53 +832,182 @@ public abstract class HttpWagonTestCase
 
     public void runTestSecuredPut( AuthenticationInfo authInfo )
         throws Exception
+    {
+        runTestSecuredPut( authInfo, 1 );
+    }
+
+    public void runTestSecuredPut( AuthenticationInfo authInfo, int putNumber )
+        throws Exception
     {
         String localRepositoryPath = FileTestUtils.getTestOutputDir().toString();
         Server server = new Server( 0 );
 
-        SecurityHandler sh = createSecurityHandler();
+        TestSecurityHandler sh = createSecurityHandler();
 
-        PutHandler handler = new PutHandler( new File( localRepositoryPath ) );
+        PutHandler putHandler = new PutHandler( new File( localRepositoryPath ) );
 
         HandlerCollection handlers = new HandlerCollection();
-        handlers.setHandlers( new Handler[] { sh, handler } );
+        handlers.setHandlers( new Handler[]{ sh, putHandler } );
 
         server.setHandler( handlers );
         addConnectors( server );
         server.start();
 
+        StreamingWagon wagon = (StreamingWagon) getWagon();
+        Repository testRepository = new Repository( "id", getRepositoryUrl( server ) );
+        wagon.connect( testRepository, authInfo );
         try
         {
-            StreamingWagon wagon = (StreamingWagon) getWagon();
-
-            Repository testRepository = new Repository( "id", getRepositoryUrl( server ) );
-
-            wagon.connect( testRepository, authInfo );
-
-            File sourceFile = new File( localRepositoryPath, "test-secured-put-resource" );
-            sourceFile.delete();
-            assertFalse( sourceFile.exists() );
-
-            File tempFile = File.createTempFile( "wagon", "tmp" );
-            tempFile.deleteOnExit();
-            FileUtils.fileWrite( tempFile.getAbsolutePath(), "put top secret" );
-
-            try
+            for ( int i = 0; i < putNumber; i++ )
             {
-                wagon.put( tempFile, "test-secured-put-resource" );
-            }
-            finally
-            {
-                wagon.disconnect();
-                tempFile.delete();
-            }
+                File sourceFile = new File( localRepositoryPath, "test-secured-put-resource" );
+                sourceFile.delete();
+                assertFalse( sourceFile.exists() );
 
-            assertEquals( "put top secret", FileUtils.fileRead( sourceFile.getAbsolutePath() ) );
+                File tempFile = File.createTempFile( "wagon", "tmp" );
+                tempFile.deleteOnExit();
+                FileUtils.fileWrite( tempFile.getAbsolutePath(), "put top secret" );
+
+                try
+                {
+                    wagon.put( tempFile, "test-secured-put-resource" );
+                }
+                finally
+                {
+                    tempFile.delete();
+                }
+
+                assertEquals( "put top secret", FileUtils.fileRead( sourceFile.getAbsolutePath() ) );
+            }
         }
         finally
         {
+            wagon.disconnect();
             server.stop();
         }
+        assertEquals( putNumber, putHandler.putCallNumber );
+        testPreemptiveAuthentication( sh );
+    }
+
+    public void testNonSecuredPutFromStream()
+        throws Exception
+    {
+        AuthenticationInfo authInfo = new AuthenticationInfo();
+        authInfo.setUserName( "user" );
+        authInfo.setPassword( "secret" );
+        runTestSecuredPutFromStream( authInfo, 1, false );
+    }
+
+    public void testSecuredPutFromStream()
+        throws Exception
+    {
+        AuthenticationInfo authInfo = new AuthenticationInfo();
+        authInfo.setUserName( "user" );
+        authInfo.setPassword( "secret" );
+        runTestSecuredPutFromStream( authInfo, 1, true );
+    }
+
+    public void runTestSecuredPutFromStream( AuthenticationInfo authInfo, int putNumber, boolean addSecurityHandler )
+        throws Exception
+    {
+        String localRepositoryPath = FileTestUtils.getTestOutputDir().toString();
+        Server server = new Server( 0 );
+
+        TestSecurityHandler sh = createSecurityHandler();
+
+        PutHandler putHandler = new PutHandler( new File( localRepositoryPath ) );
+
+        HandlerCollection handlers = new HandlerCollection();
+        handlers.setHandlers( addSecurityHandler ? new Handler[]{ sh, putHandler } : new Handler[]{ putHandler } );
+
+        server.setHandler( handlers );
+        addConnectors( server );
+        server.start();
+
+        StreamingWagon wagon = (StreamingWagon) getWagon();
+        Repository testRepository = new Repository( "id", getRepositoryUrl( server ) );
+        if ( addSecurityHandler )
+        {
+            wagon.connect( testRepository, authInfo );
+        }
+        else
+        {
+            wagon.connect( testRepository );
+        }
+        try
+        {
+            for ( int i = 0; i < putNumber; i++ )
+            {
+                File sourceFile = new File( localRepositoryPath, "test-secured-put-resource" );
+                sourceFile.delete();
+                assertFalse( sourceFile.exists() );
+
+                File tempFile = File.createTempFile( "wagon", "tmp" );
+                tempFile.deleteOnExit();
+                String content = "put top secret";
+                FileUtils.fileWrite( tempFile.getAbsolutePath(), content );
+
+                FileInputStream fileInputStream = new FileInputStream( tempFile );
+                try
+                {
+                    wagon.putFromStream( fileInputStream, "test-secured-put-resource", content.length(), -1 );
+                }
+                finally
+                {
+                    fileInputStream.close();
+                    tempFile.delete();
+
+                }
+
+                assertEquals( content, FileUtils.fileRead( sourceFile.getAbsolutePath() ) );
+            }
+        }
+        finally
+        {
+            wagon.disconnect();
+            server.stop();
+        }
+        assertEquals( putNumber, putHandler.putCallNumber );
+        if ( addSecurityHandler )
+        {
+            testPreemptiveAuthentication( sh );
+        }
+
+        // ensure we didn't use chunked transfer which doesn't work on ngnix
+        for ( DeployedResource deployedResource : putHandler.deployedResources )
+        {
+            if ( StringUtils.equalsIgnoreCase( "chunked", deployedResource.transferEncoding ) )
+            {
+                fail( "deployedResource use chunked: " + deployedResource );
+            }
+        }
+    }
+
+
+    protected abstract boolean supportPreemptiveAuthentication();
+
+    protected boolean supportProxyPreemptiveAuthentication()
+    {
+        return false;
+    }
+
+    protected void testPreemptiveAuthentication( TestSecurityHandler sh )
+    {
+
+        if ( supportPreemptiveAuthentication() )
+        {
+            assertEquals( "not 1 security handler use " + sh.securityHandlerRequestReponses, 1,
+                          sh.securityHandlerRequestReponses.size() );
+            assertEquals( 200, sh.securityHandlerRequestReponses.get( 0 ).responseCode );
+        }
+        else
+        {
+            assertEquals( "not 2 security handler use " + sh.securityHandlerRequestReponses, 2,
+                          sh.securityHandlerRequestReponses.size() );
+            assertEquals( 401, sh.securityHandlerRequestReponses.get( 0 ).responseCode );
+            assertEquals( 200, sh.securityHandlerRequestReponses.get( 1 ).responseCode );
+
+        }
     }
 
     static class StatusHandler
@@ -832,11 +1031,44 @@ public abstract class HttpWagonTestCase
         }
     }
 
+    static class DeployedResource
+    {
+        String httpMethod;
+
+        String requestUri;
+
+        String contentLength;
+
+        String transferEncoding;
+
+        public DeployedResource()
+        {
+            // no op
+        }
+
+        @Override
+        public String toString()
+        {
+            final StringBuilder sb = new StringBuilder();
+            sb.append( "DeployedResource" );
+            sb.append( "{httpMethod='" ).append( httpMethod ).append( '\'' );
+            sb.append( ", requestUri='" ).append( requestUri ).append( '\'' );
+            sb.append( ", contentLength='" ).append( contentLength ).append( '\'' );
+            sb.append( ", transferEncoding='" ).append( transferEncoding ).append( '\'' );
+            sb.append( '}' );
+            return sb.toString();
+        }
+    }
+
     static class PutHandler
         extends AbstractHandler
     {
         private final File resourceBase;
 
+        public List<DeployedResource> deployedResources = new ArrayList<DeployedResource>();
+
+        public int putCallNumber = 0;
+
         public PutHandler( File repositoryDirectory )
         {
             this.resourceBase = repositoryDirectory;
@@ -868,6 +1100,15 @@ public abstract class HttpWagonTestCase
                 in.close();
                 out.close();
             }
+            System.out.println( "put file " + request.getPathInfo() );
+            putCallNumber++;
+            DeployedResource deployedResource = new DeployedResource();
+
+            deployedResource.httpMethod = request.getMethod();
+            deployedResource.requestUri = request.getRequestURI();
+            deployedResource.transferEncoding = request.getHeader( "Transfer-Encoding" );
+            deployedResource.contentLength = request.getHeader( "Content-Length" );
+            deployedResources.add( deployedResource );
 
             response.setStatus( HttpServletResponse.SC_CREATED );
         }
@@ -876,17 +1117,24 @@ public abstract class HttpWagonTestCase
     private static class AuthorizingProxyHandler
         extends TestHeaderHandler
     {
+
+        List<SecurityHandlerRequestReponse> securityHandlerRequestReponses =
+            new ArrayList<SecurityHandlerRequestReponse>();
+
         public void handle( String target, HttpServletRequest request, HttpServletResponse response, int dispatch )
             throws IOException, ServletException
         {
+            System.out.println( " handle proxy request" );
             if ( request.getHeader( "Proxy-Authorization" ) == null )
             {
+                securityHandlerRequestReponses.add( new SecurityHandlerRequestReponse( request.getMethod(), 407 ) );
                 response.setStatus( 407 );
                 response.addHeader( "Proxy-Authenticate", "Basic realm=\"Squid proxy-caching web server\"" );
 
                 ( (Request) request ).setHandled( true );
                 return;
             }
+            securityHandlerRequestReponses.add( new SecurityHandlerRequestReponse( request.getMethod(), 200 ) );
             super.handle( target, request, response, dispatch );
         }
     }
@@ -894,7 +1142,7 @@ public abstract class HttpWagonTestCase
     private static class TestHeaderHandler
         extends AbstractHandler
     {
-        private Map headers;
+        public Map headers = Collections.EMPTY_MAP;
 
         public TestHeaderHandler()
         {
@@ -918,4 +1166,69 @@ public abstract class HttpWagonTestCase
         }
 
     }
+
+    protected TestSecurityHandler createSecurityHandler()
+    {
+        Constraint constraint = new Constraint();
+        constraint.setName( Constraint.__BASIC_AUTH );
+        constraint.setRoles( new String[]{ "admin" } );
+        constraint.setAuthenticate( true );
+
+        ConstraintMapping cm = new ConstraintMapping();
+        cm.setConstraint( constraint );
+        cm.setPathSpec( "/*" );
+
+        TestSecurityHandler sh = new TestSecurityHandler();
+        HashUserRealm hashUserRealm = new HashUserRealm( "MyRealm" );
+        hashUserRealm.put( "user", "secret" );
+        hashUserRealm.addUserToRole( "user", "admin" );
+        sh.setUserRealm( hashUserRealm );
+        sh.setConstraintMappings( new ConstraintMapping[]{ cm } );
+        return sh;
+    }
+
+    public static class TestSecurityHandler
+        extends SecurityHandler
+    {
+
+        public List<SecurityHandlerRequestReponse> securityHandlerRequestReponses =
+            new ArrayList<SecurityHandlerRequestReponse>();
+
+        @Override
+        public void handle( String target, HttpServletRequest request, HttpServletResponse response, int dispatch )
+            throws IOException, ServletException
+        {
+            String method = request.getMethod();
+            super.handle( target, request, response, dispatch );
+            System.out.println( "method in SecurityHandler: " + method );
+
+            securityHandlerRequestReponses.add(
+                new SecurityHandlerRequestReponse( method, ( (Response) response ).getStatus() ) );
+        }
+
+    }
+
+    public static class SecurityHandlerRequestReponse
+    {
+        public String method;
+
+        public int responseCode;
+
+        private SecurityHandlerRequestReponse( String method, int responseCode )
+        {
+            this.method = method;
+            this.responseCode = responseCode;
+        }
+
+        @Override
+        public String toString()
+        {
+            final StringBuilder sb = new StringBuilder();
+            sb.append( "SecurityHandlerRequestReponse" );
+            sb.append( "{method='" ).append( method ).append( '\'' );
+            sb.append( ", responseCode=" ).append( responseCode );
+            sb.append( '}' );
+            return sb.toString();
+        }
+    }
 }
