@@ -24,13 +24,13 @@ import org.apache.commons.httpclient.Header;
 import org.apache.commons.httpclient.HostConfiguration;
 import org.apache.commons.httpclient.HttpClient;
 import org.apache.commons.httpclient.HttpConnectionManager;
-import org.apache.commons.httpclient.HttpException;
 import org.apache.commons.httpclient.HttpMethod;
 import org.apache.commons.httpclient.HttpStatus;
 import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
 import org.apache.commons.httpclient.NTCredentials;
 import org.apache.commons.httpclient.UsernamePasswordCredentials;
 import org.apache.commons.httpclient.auth.AuthScope;
+import org.apache.commons.httpclient.cookie.CookiePolicy;
 import org.apache.commons.httpclient.methods.GetMethod;
 import org.apache.commons.httpclient.methods.HeadMethod;
 import org.apache.commons.httpclient.methods.PutMethod;
@@ -84,7 +84,8 @@ public abstract class AbstractHttpClientWagon
 
         private final File source;
 
-        private RequestEntityImplementation( final InputStream stream, final Resource resource, final Wagon wagon, final File source )
+        private RequestEntityImplementation( final InputStream stream, final Resource resource, final Wagon wagon,
+                                             final File source )
             throws TransferFailedException
         {
             if ( source != null )
@@ -98,7 +99,7 @@ public abstract class AbstractHttpClientWagon
                 {
                     this.source = File.createTempFile( "http-wagon.", ".tmp" );
                     this.source.deleteOnExit();
-                    
+
                     fos = new FileOutputStream( this.source );
                     IOUtil.copy( stream, fos );
                 }
@@ -112,7 +113,7 @@ public abstract class AbstractHttpClientWagon
                     IOUtil.close( fos );
                 }
             }
-            
+
             this.resource = resource;
             this.wagon = wagon;
         }
@@ -136,11 +137,11 @@ public abstract class AbstractHttpClientWagon
             throws IOException
         {
             byte[] buffer = new byte[DEFAULT_BUFFER_SIZE];
-            
+
             TransferEvent transferEvent =
                 new TransferEvent( wagon, resource, TransferEvent.TRANSFER_PROGRESS, TransferEvent.REQUEST_PUT );
             transferEvent.setTimestamp( System.currentTimeMillis() );
-            
+
             FileInputStream fin = null;
             try
             {
@@ -149,16 +150,16 @@ public abstract class AbstractHttpClientWagon
                 while ( remaining > 0 )
                 {
                     int n = fin.read( buffer, 0, Math.min( buffer.length, remaining ) );
-                
+
                     if ( n == -1 )
                     {
                         break;
                     }
-                
+
                     fireTransferProgress( transferEvent, buffer, n );
-                
+
                     output.write( buffer, 0, n );
-                
+
                     remaining -= n;
                 }
             }
@@ -166,7 +167,7 @@ public abstract class AbstractHttpClientWagon
             {
                 IOUtil.close( fin );
             }
-            
+
             output.flush();
         }
     }
@@ -183,7 +184,7 @@ public abstract class AbstractHttpClientWagon
      * @deprecated Use httpConfiguration instead.
      */
     private Properties httpHeaders;
-    
+
     /**
      * @since 1.0-beta-6
      */
@@ -195,6 +196,10 @@ public abstract class AbstractHttpClientWagon
     {
         repository.setUrl( getURL( repository ) );
         client = new HttpClient( connectionManager );
+
+        // WAGON-273: default the cookie-policy to browser compatible
+        client.getParams().setCookiePolicy( CookiePolicy.BROWSER_COMPATIBILITY );
+
         String username = null;
         String password = null;
 
@@ -203,6 +208,8 @@ public abstract class AbstractHttpClientWagon
             username = authenticationInfo.getUserName();
 
             password = authenticationInfo.getPassword();
+
+            client.getParams().setAuthenticationPreemptive( true );
         }
 
         String host = getRepository().getHost();
@@ -212,7 +219,7 @@ public abstract class AbstractHttpClientWagon
             Credentials creds = new UsernamePasswordCredentials( username, password );
 
             int port = getRepository().getPort() > -1 ? getRepository().getPort() : AuthScope.ANY_PORT;
-            
+
             AuthScope scope = new AuthScope( host, port );
             client.getState().setCredentials( scope, creds );
         }
@@ -245,7 +252,7 @@ public abstract class AbstractHttpClientWagon
                     }
 
                     int port = proxyInfo.getPort() > -1 ? proxyInfo.getPort() : AuthScope.ANY_PORT;
-                    
+
                     AuthScope scope = new AuthScope( proxyHost, port );
                     client.getState().setProxyCredentials( scope, creds );
                 }
@@ -270,40 +277,44 @@ public abstract class AbstractHttpClientWagon
         throws TransferFailedException, ResourceDoesNotExistException, AuthorizationException
     {
         Resource resource = new Resource( resourceName );
-        
+
         firePutInitiated( resource, source );
-        
+
         resource.setContentLength( source.length() );
-        
+
         resource.setLastModified( source.lastModified() );
 
         put( null, resource, source );
     }
-    
+
     public void putFromStream( final InputStream stream, String destination, long contentLength, long lastModified )
         throws TransferFailedException, ResourceDoesNotExistException, AuthorizationException
     {
         Resource resource = new Resource( destination );
-        
+
         firePutInitiated( resource, null );
-        
+
         resource.setContentLength( contentLength );
-        
+
         resource.setLastModified( lastModified );
-        
+
         put( stream, resource, null );
     }
 
     private void put( final InputStream stream, Resource resource, File source )
         throws TransferFailedException, AuthorizationException, ResourceDoesNotExistException
     {
-        String url = getRepository().getUrl();
+        StringBuilder url = new StringBuilder( getRepository().getUrl() );
         String[] parts = StringUtils.split( resource.getName(), "/" );
-        for ( int i = 0; i < parts.length; i++ )
+        for ( String part : parts )// int i = 0; i < parts.length; i++ )
         {
             // TODO: Fix encoding...
             // url += "/" + URLEncoder.encode( parts[i], System.getProperty("file.encoding") );
-            url += "/" + URLEncoder.encode( parts[i] );
+            if ( !url.toString().endsWith( "/" ) )
+            {
+                url.append( '/' );
+            }
+            url.append( URLEncoder.encode( part ) );
         }
 
         //Parent directories need to be created before posting
@@ -316,10 +327,10 @@ public abstract class AbstractHttpClientWagon
             fireTransferError( resource, e, TransferEvent.REQUEST_GET );
         }
 
-        PutMethod putMethod = new PutMethod( url );
+        PutMethod putMethod = new PutMethod( url.toString() );
 
         firePutStarted( resource, source );
-                
+
         try
         {
             putMethod.setRequestEntity( new RequestEntityImplementation( stream, resource, this, source ) );
@@ -362,12 +373,11 @@ public abstract class AbstractHttpClientWagon
                 case HttpStatus.SC_NOT_FOUND:
                     throw new ResourceDoesNotExistException( "File: " + url + " does not exist" );
 
-                //add more entries here
-                default :
+                    //add more entries here
+                default:
                 {
-                    TransferFailedException e =
-                            new TransferFailedException( "Failed to transfer file: " + url + ". Return code is: "
-                                + statusCode );
+                    TransferFailedException e = new TransferFailedException(
+                        "Failed to transfer file: " + url + ". Return code is: " + statusCode );
                     fireTransferError( resource, e, TransferEvent.REQUEST_PUT );
                     throw e;
                 }
@@ -380,16 +390,23 @@ public abstract class AbstractHttpClientWagon
             putMethod.releaseConnection();
         }
     }
-    
-    protected void mkdirs( String dirname ) throws HttpException, IOException
+
+    protected void mkdirs( String dirname )
+        throws IOException
     {
+        // do nothing as default.
     }
 
     public boolean resourceExists( String resourceName )
         throws TransferFailedException, AuthorizationException
     {
-        String url = getRepository().getUrl() + "/" + resourceName;
-        HeadMethod headMethod = new HeadMethod( url );
+        StringBuilder url = new StringBuilder( getRepository().getUrl() );
+        if ( !url.toString().endsWith( "/" ) )
+        {
+            url.append( '/' );
+        }
+        url.append( resourceName );
+        HeadMethod headMethod = new HeadMethod( url.toString() );
         int statusCode;
         try
         {
@@ -424,10 +441,10 @@ public abstract class AbstractHttpClientWagon
                 case HttpStatus.SC_NOT_FOUND:
                     return false;
 
-                    //add more entries here
+                //add more entries here
                 default:
-                    throw new TransferFailedException( "Failed to transfer file: " + url + ". Return code is: "
-                        + statusCode );
+                    throw new TransferFailedException(
+                        "Failed to transfer file: " + url + ". Return code is: " + statusCode );
             }
         }
         finally
@@ -436,20 +453,22 @@ public abstract class AbstractHttpClientWagon
         }
     }
 
-    protected int execute( HttpMethod httpMethod ) throws HttpException, IOException
+    protected int execute( HttpMethod httpMethod )
+        throws IOException
     {
-        int statusCode = SC_NULL;
-        
+        int statusCode;
+
         setParameters( httpMethod );
         setHeaders( httpMethod );
-        
+
         statusCode = client.executeMethod( httpMethod );
         return statusCode;
     }
-    
+
     protected void setParameters( HttpMethod method )
     {
-        HttpMethodConfiguration config = httpConfiguration == null ? null : httpConfiguration.getMethodConfiguration( method );
+        HttpMethodConfiguration config =
+            httpConfiguration == null ? null : httpConfiguration.getMethodConfiguration( method );
         if ( config != null )
         {
             HttpMethodParams params = config.asMethodParams( method.getParams() );
@@ -458,7 +477,7 @@ public abstract class AbstractHttpClientWagon
                 method.setParams( params );
             }
         }
-        
+
         if ( config == null || config.getConnectionTimeout() == HttpMethodConfiguration.DEFAULT_CONNECTION_TIMEOUT )
         {
             method.getParams().setSoTimeout( getTimeout() );
@@ -467,7 +486,8 @@ public abstract class AbstractHttpClientWagon
 
     protected void setHeaders( HttpMethod method )
     {
-        HttpMethodConfiguration config = httpConfiguration == null ? null : httpConfiguration.getMethodConfiguration( method );
+        HttpMethodConfiguration config =
+            httpConfiguration == null ? null : httpConfiguration.getMethodConfiguration( method );
         if ( config == null || config.isUseDefaultHeaders() )
         {
             // TODO: merge with the other headers and have some better defaults, unify with lightweight headers
@@ -484,9 +504,9 @@ public abstract class AbstractHttpClientWagon
             {
                 String header = (String) i.next();
                 method.addRequestHeader( header, httpHeaders.getProperty( header ) );
-            }                
+            }
         }
-        
+
         Header[] headers = config == null ? null : config.asRequestHeaders();
         if ( headers != null )
         {
@@ -500,6 +520,7 @@ public abstract class AbstractHttpClientWagon
     /**
      * getUrl
      * Implementors can override this to remove unwanted parts of the url such as role-hints
+     *
      * @param repository
      * @return
      */
@@ -542,9 +563,15 @@ public abstract class AbstractHttpClientWagon
         throws TransferFailedException, ResourceDoesNotExistException, AuthorizationException
     {
         Resource resource = inputData.getResource();
-        
-        String url = getRepository().getUrl() + "/" + resource.getName();
-        getMethod = new GetMethod( url );
+
+        StringBuilder url = new StringBuilder( getRepository().getUrl() );
+        if ( !url.toString().endsWith( "/" ) )
+        {
+            url.append( '/' );
+        }
+        url.append( resource.getName() );
+
+        getMethod = new GetMethod( url.toString() );
         long timestamp = resource.getLastModified();
         if ( timestamp > 0 )
         {
@@ -606,9 +633,8 @@ public abstract class AbstractHttpClientWagon
             default:
             {
                 cleanupGetTransfer( resource );
-                TransferFailedException e =
-                    new TransferFailedException( "Failed to transfer file: " + url + ". Return code is: "
-                        + statusCode );
+                TransferFailedException e = new TransferFailedException(
+                    "Failed to transfer file: " + url + ". Return code is: " + statusCode );
                 fireTransferError( resource, e, TransferEvent.REQUEST_GET );
                 throw e;
             }
@@ -628,8 +654,8 @@ public abstract class AbstractHttpClientWagon
             }
             catch ( NumberFormatException e )
             {
-                fireTransferDebug( "error parsing content length header '" + contentLengthHeader.getValue() + "' "
-                    + e );
+                fireTransferDebug(
+                    "error parsing content length header '" + contentLengthHeader.getValue() + "' " + e );
             }
         }
 
@@ -654,8 +680,7 @@ public abstract class AbstractHttpClientWagon
         }
 
         Header contentEncoding = getMethod.getResponseHeader( "Content-Encoding" );
-        boolean isGZipped =
-            contentEncoding == null ? false : "gzip".equalsIgnoreCase( contentEncoding.getValue() );
+        boolean isGZipped = contentEncoding != null && "gzip".equalsIgnoreCase( contentEncoding.getValue() );
 
         try
         {
@@ -671,10 +696,10 @@ public abstract class AbstractHttpClientWagon
 
             String msg =
                 "Error occurred while retrieving from remote repository:" + getRepository() + ": " + e.getMessage();
-            
+
             throw new TransferFailedException( msg, e );
         }
-        
+
         inputData.setInputStream( is );
     }
 
@@ -686,9 +711,25 @@ public abstract class AbstractHttpClientWagon
         }
     }
 
+    @Override
+    public void putFromStream( InputStream stream, String destination )
+        throws TransferFailedException, ResourceDoesNotExistException, AuthorizationException
+    {
+        putFromStream( stream, destination, -1, -1 );
+    }
+
+    @Override
+    protected void putFromStream( InputStream stream, Resource resource )
+        throws TransferFailedException, AuthorizationException, ResourceDoesNotExistException
+    {
+        putFromStream( stream, resource.getName( ), -1, -1 );
+    }
+
+    @Override
     public void fillOutputData( OutputData outputData )
         throws TransferFailedException
     {
-        throw new IllegalStateException( "Should not be using the streaming wagon for HTTP PUT" );        
+        // no needed in this implementation but throw an Exception if used
+        throw new IllegalStateException( "this wagon http client must not use fillOutputData" );
     }
 }
